@@ -19,7 +19,7 @@
  * Copyright 2011 - 2013 Red Hat, Inc.
  */
 
-#include "config.h"
+#include "nm-default.h"
 
 #include <stdlib.h>
 
