@@ -19,7 +19,7 @@
  * Copyright 2014 Red Hat, Inc.
  */
 
-#include "config.h"
+#include "nm-default.h"
 
 #include <string.h>
 
@@ -58,14 +58,12 @@ NM_SETTING_REGISTER_TYPE (NM_TYPE_SETTING_IP4_CONFIG)
 
 typedef struct {
 	char *dhcp_client_id;
-	int dhcp_timeout;
 	char *dhcp_fqdn;
 } NMSettingIP4ConfigPrivate;
 
 enum {
 	PROP_0,
 	PROP_DHCP_CLIENT_ID,
-	PROP_DHCP_TIMEOUT,
 	PROP_DHCP_FQDN,
 
 	LAST_PROP
@@ -102,26 +100,6 @@ nm_setting_ip4_config_get_dhcp_client_id (NMSettingIP4Config *setting)
 	return NM_SETTING_IP4_CONFIG_GET_PRIVATE (setting)->dhcp_client_id;
 }
 
-/**
- * nm_setting_ip4_config_get_dhcp_timeout:
- * @setting: the #NMSettingIP4Config
- *
- * Returns the value contained in the #NMSettingIP4Config:dhcp-timeout
- * property.
- *
- * Returns: the configured DHCP timeout in seconds. 0 = default for
- * the particular kind of device.
- *
- * Since: 1.2
- **/
-int
-nm_setting_ip4_config_get_dhcp_timeout (NMSettingIP4Config *setting)
-{
-	g_return_val_if_fail (NM_IS_SETTING_IP4_CONFIG (setting), 0);
-
-	return NM_SETTING_IP4_CONFIG_GET_PRIVATE (setting)->dhcp_timeout;
-}
-
 /**
  * nm_setting_ip4_config_get_dhcp_fqdn:
  * @setting: the #NMSettingIP4Config
@@ -277,9 +255,6 @@ set_property (GObject *object, guint prop_id,
 		g_free (priv->dhcp_client_id);
 		priv->dhcp_client_id = g_value_dup_string (value);
 		break;
-	case PROP_DHCP_TIMEOUT:
-		priv->dhcp_timeout = g_value_get_uint (value);
-		break;
 	case PROP_DHCP_FQDN:
 		g_free (priv->dhcp_fqdn);
 		priv->dhcp_fqdn = g_value_dup_string (value);
@@ -300,9 +275,6 @@ get_property (GObject *object, guint prop_id,
 	case PROP_DHCP_CLIENT_ID:
 		g_value_set_string (value, nm_setting_ip4_config_get_dhcp_client_id (s_ip4));
 		break;
-	case PROP_DHCP_TIMEOUT:
-		g_value_set_uint (value, nm_setting_ip4_config_get_dhcp_timeout (s_ip4));
-		break;
 	case PROP_DHCP_FQDN:
 		g_value_set_string (value, nm_setting_ip4_config_get_dhcp_fqdn (s_ip4));
 		break;
@@ -682,25 +654,23 @@ nm_setting_ip4_config_class_init (NMSettingIP4ConfigClass *ip4_class)
 		                      G_PARAM_READWRITE |
 		                      G_PARAM_STATIC_STRINGS));
 
-	/**
-	 * NMSettingIP4Config:dhcp-timeout:
-	 *
-	 * A timeout for a DHCP transaction in seconds.
-	 **/
 	/* ---ifcfg-rh---
+	 * property: dad-timeout
+	 * variable: ARPING_WAIT
+	 * default: missing variable means global default (config override or 3)
+	 * description: Timeout (in seconds) for performing DAD before configuring
+	 * IPv4 addresses. 0 turns off the DAD completely, -1 means default value.
+	 * example: ARPING_WAIT=2
+	 * ---end---
+	 */
+
+	/* ---ifcfg-rh---
 	 * property: dhcp-timeout
-	 * variable: DHCP_TIMEOUT(+)
+	 * variable: IPV4_DHCP_TIMEOUT(+)
 	 * description: A timeout after which the DHCP transaction fails in case of no response.
-	 * example: DHCP_TIMEOUT=10
+	 * example: IPV4_DHCP_TIMEOUT=10
 	 * ---end---
 	 */
-	g_object_class_install_property
-		(object_class, PROP_DHCP_TIMEOUT,
-                 g_param_spec_uint (NM_SETTING_IP4_CONFIG_DHCP_TIMEOUT, "", "",
-                                    0, G_MAXUINT32, 0,
-                                    G_PARAM_READWRITE |
-                                    NM_SETTING_PARAM_FUZZY_IGNORE |
-                                    G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * NMSettingIP4Config:dhcp-fqdn:
