@@ -19,14 +19,14 @@
  * Copyright 2007 - 2014 Red Hat, Inc.
  */
 
-#include "config.h"
+#include "nm-default.h"
+
+#include "nm-setting-ip6-config.h"
 
 #include <string.h>
 
-#include "nm-setting-ip6-config.h"
 #include "nm-setting-private.h"
 #include "nm-core-enum-types.h"
-#include "nm-macros-internal.h"
 
 /**
  * SECTION:nm-setting-ip6-config
