@@ -19,7 +19,7 @@
  * Copyright 2007 - 2014 Red Hat, Inc.
  */
 
-#include "config.h"
+#include "nm-default.h"
 
 #include "nm-simple-connection.h"
 #include "nm-setting-private.h"
