@@ -19,12 +19,11 @@
  * Copyright 2007 - 2014 Red Hat, Inc.
  */
 
-#include "config.h"
+#include "nm-default.h"
 
 #include <string.h>
-#include <nm-utils.h>
 
-#include "nm-default.h"
+#include "nm-utils.h"
 #include "nm-client.h"
 #include "nm-manager.h"
 #include "nm-remote-settings.h"
