@@ -21,11 +21,11 @@
 #ifndef __MAIN_UTILS_H__
 #define __MAIN_UTILS_H__
 
-#include <glib.h>
+#include "nm-default.h"
 
 void     nm_main_utils_ensure_root (void);
 
-gboolean nm_main_utils_setup_signals (GMainLoop *main_loop, gboolean *quit_early_ptr);
+void     nm_main_utils_setup_signals (GMainLoop *main_loop);
 
 void     nm_main_utils_ensure_rundir (void);
 
@@ -37,7 +37,13 @@ gboolean nm_main_utils_early_setup (const char *progname,
                                     int *argc,
                                     char **argv[],
                                     GOptionEntry *options,
-                                    GOptionEntry *more_options,
+                                    void (*option_context_hook) (gpointer user_data, GOptionContext *opt_ctx),
+                                    gpointer option_context_hook_data,
                                     const char *summary);
 
+/* The following functions are not implemented inside nm-main-utils.c, instead
+ * main.c and nm-iface-helper.c */
+
+void nm_main_config_reload (int signal);
+
 #endif /* __MAIN_UTILS_H__ */
