@@ -18,10 +18,9 @@
  * Copyright (C) 2014 Red Hat, Inc.
  */
 
-#include <glib-object.h>
 
+#include "nm-default.h"
 #include "nm-connection.h"
-#include "nm-types.h"
 
 #ifndef __NETWORKMANAGER_DEFAULT_ROUTE_MANAGER_H__
 #define __NETWORKMANAGER_DEFAULT_ROUTE_MANAGER_H__
