@@ -18,13 +18,12 @@
  * Copyright (C) 2011 - 2015 Red Hat, Inc.
  */
 
-#include "config.h"
+#include "nm-default.h"
+
+#include "nm-firewall-manager.h"
 
 #include <string.h>
 
-#include "nm-default.h"
-#include "nm-firewall-manager.h"
-#include "gsystem-local-alloc.h"
 #include "NetworkManagerUtils.h"
 
 #define NM_FIREWALL_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), \
