@@ -21,11 +21,10 @@
 #ifndef __NM_PLATFORM_UTILS_H__
 #define __NM_PLATFORM_UTILS_H__
 
-#include "config.h"
-
 #include <gudev/gudev.h>
 
 #include "nm-platform.h"
+#include "nm-setting-wired.h"
 
 
 const char *nmp_utils_ethtool_get_driver (const char *ifname);
@@ -33,6 +32,9 @@ gboolean nmp_utils_ethtool_supports_carrier_detect (const char *ifname);
 gboolean nmp_utils_ethtool_supports_vlans (const char *ifname);
 int nmp_utils_ethtool_get_peer_ifindex (const char *ifname);
 gboolean nmp_utils_ethtool_get_wake_on_lan (const char *ifname);
+gboolean nmp_utils_ethtool_set_wake_on_lan (const char *ifname, NMSettingWiredWakeOnLan wol,
+                                            const char *wol_password);
+
 gboolean nmp_utils_ethtool_get_link_speed (const char *ifname, guint32 *out_speed);
 
 gboolean nmp_utils_ethtool_get_driver_info (const char *ifname,
@@ -63,4 +65,8 @@ gboolean nmp_utils_lifetime_get (guint32 timestamp,
                                  guint32 *out_lifetime,
                                  guint32 *out_preferred);
 
+gboolean nmp_utils_device_exists (const char *name);
+
+gboolean nmp_utils_ip4_address_is_link_local (in_addr_t addr);
+
 #endif /* __NM_PLATFORM_UTILS_H__ */
