@@ -18,16 +18,17 @@
  * Copyright (C) 2015 Red Hat, Inc.
  */
 
-#include "config.h"
+#include "nm-default.h"
+
+#include "nmp-object.h"
 
 #include <unistd.h>
 
-#include "nm-default.h"
-#include "nmp-object.h"
-#include "nm-platform-utils.h"
-#include "NetworkManagerUtils.h"
 #include "nm-utils.h"
 
+#include "nm-core-utils.h"
+#include "nm-platform-utils.h"
+
 /*********************************************************************************************/
 
 #define _NMLOG_DOMAIN LOGD_PLATFORM
@@ -1256,7 +1257,7 @@ nmp_cache_link_connected_needs_toggle (const NMPCache *cache, const NMPObject *m
 
 	/* if native IFF_LOWER_UP is down, link.connected must also be down
 	 * regardless of the slaves. */
-	if (!NM_FLAGS_HAS (master->link.flags, IFF_LOWER_UP))
+	if (!NM_FLAGS_HAS (master->link.n_ifi_flags, IFF_LOWER_UP))
 		return !!master->link.connected;
 
 	if (potential_slave && NMP_OBJECT_GET_TYPE (potential_slave) != NMP_OBJECT_TYPE_LINK)
