@@ -28,7 +28,7 @@
 
 #include <nm-connection.h>
 
-#include "nm-types.h"
+#include "nm-exported-object.h"
 
 #define NM_TYPE_SETTINGS            (nm_settings_get_type ())
 #define NM_SETTINGS(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), NM_TYPE_SETTINGS, NMSettings))
@@ -51,11 +51,11 @@
 #define NM_SETTINGS_SIGNAL_AGENT_REGISTERED              "agent-registered"
 
 struct _NMSettings {
-	GObject parent_instance;
+	NMExportedObject parent_instance;
 };
 
 typedef struct {
-	GObjectClass parent_class;
+	NMExportedObjectClass parent_class;
 
 	/* Signals */
 	void (*properties_changed) (NMSettings *self, GHashTable *properties);
@@ -73,7 +73,8 @@ typedef struct {
 
 GType nm_settings_get_type (void);
 
-NMSettings *nm_settings_new (GError **error);
+NMSettings *nm_settings_new (void);
+gboolean nm_settings_start (NMSettings *self, GError **error);
 
 typedef void (*NMSettingsForEachFunc) (NMSettings *settings,
                                        NMSettingsConnection *connection,
@@ -86,13 +87,14 @@ void nm_settings_for_each_connection (NMSettings *settings,
 typedef void (*NMSettingsAddCallback) (NMSettings *settings,
                                        NMSettingsConnection *connection,
                                        GError *error,
-                                       DBusGMethodInvocation *context,
+                                       GDBusMethodInvocation *context,
+                                       NMAuthSubject *subject,
                                        gpointer user_data);
 
 void nm_settings_add_connection_dbus (NMSettings *self,
                                       NMConnection *connection,
                                       gboolean save_to_disk,
-                                      DBusGMethodInvocation *context,
+                                      GDBusMethodInvocation *context,
                                       NMSettingsAddCallback callback,
                                       gpointer user_data);
 
@@ -111,6 +113,8 @@ NMSettingsConnection *nm_settings_get_connection_by_path (NMSettings *settings,
 NMSettingsConnection *nm_settings_get_connection_by_uuid (NMSettings *settings,
                                                           const char *uuid);
 
+gboolean nm_settings_has_connection (NMSettings *self, NMSettingsConnection *connection);
+
 const GSList *nm_settings_get_unmanaged_specs (NMSettings *self);
 
 char *nm_settings_get_hostname (NMSettings *self);
