@@ -19,17 +19,17 @@
  * Copyright (C) 1999-2010 Gentoo Foundation, Inc.
  */
 
-#include "config.h"
+#include "nm-default.h"
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
-#include <nm-utils.h>
+
+#include "nm-utils.h"
 #include "NetworkManagerUtils.h"
-#include <nm-settings-plugin.h>
+#include "nm-settings-plugin.h"
 #include "nm-config.h"
-#include "nm-default.h"
 #include "net_utils.h"
 #include "wpa_parser.h"
 #include "net_parser.h"
@@ -712,7 +712,7 @@ _has_prefix_impl (char *str, const char *prefix, gsize prefix_len)
 	} while (g_ascii_isspace (str[0]));
 	return str;
 }
-#define _has_prefix(STR, PREFIX) _has_prefix_impl (STR, PREFIX, STRLEN (PREFIX))
+#define _has_prefix(STR, PREFIX) _has_prefix_impl (STR, PREFIX, NM_STRLEN (PREFIX))
 
 void
 get_dhcp_hostname_and_client_id (char **hostname, char **client_id)
